package xyz.lychee.lagfixer.libs.adventure.platform.bukkit;

import java.util.Collection;
import java.util.function.Supplier;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.Server;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.Facet;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.FacetComponentFlattener;
import xyz.lychee.lagfixer.libs.adventure.text.flattener.ComponentFlattener;
import xyz.lychee.lagfixer.libs.adventure.text.serializer.gson.GsonComponentSerializer;
import xyz.lychee.lagfixer.libs.adventure.text.serializer.gson.legacyimpl.NBTLegacyHoverEventSerializer;
import xyz.lychee.lagfixer.libs.adventure.text.serializer.legacy.LegacyComponentSerializer;
import xyz.lychee.lagfixer.libs.annotations.NotNull;

public final class BukkitComponentSerializer {
   private static final boolean IS_1_16 = MinecraftReflection.findEnum(Material.class, "NETHERITE_PICKAXE") != null;
   private static final Collection<FacetComponentFlattener.Translator<Server>> TRANSLATORS = Facet.of(new Supplier[]{SpigotFacet.Translator::new, CraftBukkitFacet.Translator::new});
   private static final LegacyComponentSerializer LEGACY_SERIALIZER;
   private static final GsonComponentSerializer GSON_SERIALIZER;
   static final ComponentFlattener FLATTENER;

   private BukkitComponentSerializer() {
   }

   public static @NotNull LegacyComponentSerializer legacy() {
      return LEGACY_SERIALIZER;
   }

   public static @NotNull GsonComponentSerializer gson() {
      return GSON_SERIALIZER;
   }

   static {
      FLATTENER = FacetComponentFlattener.get(Bukkit.getServer(), TRANSLATORS);
      if (IS_1_16) {
         LEGACY_SERIALIZER = LegacyComponentSerializer.builder().hexColors().useUnusualXRepeatedCharacterHexFormat().flattener(FLATTENER).build();
         GSON_SERIALIZER = GsonComponentSerializer.builder().legacyHoverEventSerializer(NBTLegacyHoverEventSerializer.get()).build();
      } else {
         LEGACY_SERIALIZER = LegacyComponentSerializer.builder().character('§').flattener(FLATTENER).build();
         GSON_SERIALIZER = GsonComponentSerializer.builder().legacyHoverEventSerializer(NBTLegacyHoverEventSerializer.get()).emitLegacyHoverEvent().downsampleColors().build();
      }

   }
}
